% ===============================================================
% Title  : Matlab_M_Sim.m
% Author : Arend Aerts
% Project: Model-Based testing tool for hybrid systems in Acumen
% Date   : 12-03-2015
% Description: Simulate hybrid system model in MATLAB (continious/discrete)
% ===============================================================

% Hybrid system CONTINIOUS time simulation 

 % STATUS UPDATE GUI 
disp(strcat(datestr(now),'   [Matlab model] START'));

% INIT
SimStep= str2double(handles.acumen_sim_var{1,1});   %seconds
SimTime = str2double(handles.acumen_sim_var{2,1});

% Model parameters
t_min  = 20;
t_max = 25;
t_delta = 3;
sysd1 = c2d(ss(-1,(t_max+t_delta),1,0),SimStep);
sysd2 = c2d(ss(-1,(t_min-t_delta),1,0),SimStep);

x0 = 0;
mode = 1;

% LOG
mod = 1;
x=x0;
t=0;
tmp=x0;
i=2;

% CALC
for(k=SimStep:SimStep:SimTime)
    if (mode == 1) & (tmp >= t_max)
        mode = 2;
    else
        if (mode == 2) & (tmp <= t_min)
            mode = 1;
        end   
    end
    
    if mode == 1
        tmp = sysd1.a*tmp+sysd1.b;
    end
    if mode == 2
        tmp = sysd2.a*tmp+sysd2.b;
    end
    
    x(1,i)=tmp;
    t(1,i)=k;
    mod(1,i)=mode;
    i=i+1;
end

% Export data
handles.Matlab_M(1,:) = t;
handles.Matlab_M(2,:) = x;
handles.Matlab_M(3,:) = mod;


%Plot
% figure;
% plot(t,x,'b--','linewidth',2)
% hold on;
% plot(t,mod,'k--','linewidth',2)
% grid on;
% title('Matlab model simulation (discrete)');
% xlabel('Time');
% ylabel('Temperature');
% legend('Output','state');


% STATUS UPDATE GUI 
disp(strcat(datestr(now),'   [Matlab model] FINISH'));


