% =========================================================================
% Title         : TestCaseGeneration_BoostConverter.m
% Author        : Arend Aerts
% Project       : Model-Based testing tool for hybrid systems in Acumen
% Date          : 10-04-2015
% Description   : Test case generation for conformance testing 
%
%                                ---
%
% Copyright () 2010-2014, Acumen Individual Contributors (AIC)
% All rights reserved.
%
% See LICENSE-AIC for licensing details. 
% =========================================================================

%function [testcase] = TestCaseGeneration_BoostConverter(acumen,conformance)

% User interface connection
acumen = handles.acumen;
conformance = handles.conformance;


% MODEL DECLARATION --
% Global model parameters
Ein = 12; 
Iin = 0;
Pin = 0;
Eout = 0;
Iout = 0;
Pout = 0;
S = 0;
R = 20;
C = 0.000042;
L = 0.000080;
q_init = 0;
p_init = 0;
mode = 1;

% Mode dependent parameters 
% mode q1
q1.dynamics = c2d(ss([-1/(R*C) 1/L; -1/C 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);        

% mode q2
q2.dynamics = c2d(ss([-1/(R*C) 0; 0 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);   

% mode q3
q3.dynamics = c2d(ss([-1/(R*C) 0; 0 0],[0 ; 0],[1 0 ; 0 1],[0;0]),acumen.timestep);        

% mode q4
q4.dynamics = c2d(ss([0 0; 0 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);


% Init test case generation --
state = [q_init ; p_init];      
i = 1;
k = 0;

% Test case generation algorithm --
while (k <= acumen.endtime)
        
    % Generate input sample 
     if k >= 0.0033 && R ~= 5 && R ~= 25
        R = 5;
        % mode q1
        q1.dynamics = c2d(ss([-1/(R*C) 1/L; -1/C 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);        

        % mode q2
        q2.dynamics = c2d(ss([-1/(R*C) 0; 0 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);   

        % mode q3
        q3.dynamics = c2d(ss([-1/(R*C) 0; 0 0],[0 ; 0],[1 0 ; 0 1],[0;0]),acumen.timestep);        

        % mode q4
        q4.dynamics = c2d(ss([0 0; 0 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);
    end
    
    if k >= 0.0066 && R ~= 25
        R = 25;
        % mode q1
        q1.dynamics = c2d(ss([-1/(R*C) 1/L; -1/C 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);        

        % mode q2
        q2.dynamics = c2d(ss([-1/(R*C) 0; 0 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);   

        % mode q3
        q3.dynamics = c2d(ss([-1/(R*C) 0; 0 0],[0 ; 0],[1 0 ; 0 1],[0;0]),acumen.timestep);        

        % mode q4
        q4.dynamics = c2d(ss([0 0; 0 0],[0 ; 1],[1 0 ; 0 1],[0;0]),acumen.timestep);
    end
    
    % Update testcase structure
    testcase.data(1,i) = Ein;
    testcase.data(2,i) = Iin;
    testcase.data(3,i) = Pin;
    testcase.data(4,i) = Eout;
    testcase.data(5,i) = Iout;
    testcase.data(6,i) = Pout;
    testcase.data(7,i) = mode;
    testcase.data(8,i) = R;
    testcase.data(9,i) = S;
    testcase.data(10,i) = state(1,1);
    testcase.data(11,i) = state(2,1);
    testcase.time(1,i) = k;
    testcase.var = {'Ein' 'Iin' 'Pin' 'Eout' 'Iout' 'Pout' 'mode' 'R' 'S' 'q' 'p'};
    testcase.inputvalue(1,i) = R;
    testcase.inputvar = {'R'};    
    
    % Apply input sample and determine output 
    switch mode
        case 1
            state = q1.dynamics.a * state + q1.dynamics.b * Ein;
        case 2
            state = q2.dynamics.a * state + q2.dynamics.b * Ein;
        case 3
            state = q3.dynamics.a * state + q3.dynamics.b * Ein;
        case 4
            state = q4.dynamics.a * state + q4.dynamics.b * Ein;
    end
    
    % Check guards / update mode 
    switch mode
        case 1
            if S == 1 && state(1,1) > 0
                mode = 2;
            else
                if S == 1 && state(1,1) < 0
                    mode = 4;
                    state(1,1) = 0;
                else
                    if state(2,1) < 0
                        mode = 3;
                        state(2,1) = 0;
                    end
                end
            end
        case 2
            if S == 0 && state(2,1) > 0
                mode = 1;
            else
                if S == 0 && state(2,1) < 0
                    mode = 3;
                    state(2,1) = 0;
                else
                    if state(1,1) < 0
                        mode = 4;
                        state(1,1) = 0;
                    end
                end
            end         
        case 3
            if S == 1 && state(1,1) >= 0
                mode = 2;
            else
                if S == 1 && state(1,1) < 0
                    mode = 4;
                    state(1,1) = 0;
                else
                    if state(1,1) <= (Ein*C)
                        mode = 1;
                    end
                end
            end           
        case 4
            if S == 0 && state(2,1) >= 0
                mode = 1;
            else
                if S == 0 && state(2,1) < 0
                    mode = 3;
                    state(2,1) = 0;
                else
                    if state(1,1) < 0
                        mode = 4;
                        state(1,1) = 0;
                    end
                end
            end             
    end    
    
    % Save current state
    state_old = state;
    mode_old = mode;
    
    % Sensitivity check                             % HARDCODED AND INSUFFICIENT!!!
    conformance.var = {'q' 'p'};                    % BUGFIX!!!
    conformance.epsilon = [0 0];    % BUGFIX!!!
    for m=1:1:2
        switch m
            case 1
               state = state + [0 ; conformance.epsilon(1,2)]; 
            case 2
               state = state - [0 ; conformance.epsilon(1,2)];  
            case 3
                state = state + [conformance.epsilon(1,1) ; 0]; 
            case 4
                state = state - [conformance.epsilon(1,1) ; 0]; 
        end
        
           % Check guards / update mode 
    switch mode
        case 1
            if S == 1 && state(1,1) > 0
                mode = 2;
            else
                if S == 1 && state(1,1) < 0
                    mode = 4;
                    state(1,1) = 0;
                else
                    if state(2,1) < 0
                        mode = 3;
                        state(2,1) = 0;
                    end
                end
            end
        case 2
            if S == 0 && state(2,1) > 0
                mode = 1;
            else
                if S == 0 && state(2,1) < 0
                    mode = 3;
                    state(2,1) = 0;
                else
                    if state(1,1) < 0
                        mode = 4;
                        state(1,1) = 0;
                    end
                end
            end         
        case 3
            if S == 1 && state(1,1) >= 0
                mode = 2;
            else
                if S == 1 && state(1,1) < 0
                    mode = 4;
                    state(1,1) = 0;
                else
                    if state(1,1) <= (Ein*C)
                        mode = 1;
                    end
                end
            end           
        case 4
            if S == 0 && state(2,1) >= 0
                mode = 1;
            else
                if S == 0 && state(2,1) < 0
                    mode = 3;
                    state(2,1) = 0;
                else
                    if state(1,1) < 0
                        mode = 4;
                        state(1,1) = 0;
                    end
                end
            end   
         end
    end
    
    
   if mode ~= mode_old
       mode = mode_old;
       state = state_old;
   else
        % Update variables
        Eout = state(1,1) / C;
        Iout = Eout/R;
        Pout = Iout*Eout;
        Iin  = state(2,1) / L;
        Pin  = Ein * Iin;
        
        if sin(1000000*(i-1)) >= 0 
            S = 1;
        else
            S = 0;
        end

        k = k + acumen.timestep;
        i = i + 1;
    end
end

% User interface connection
handles.testcase = testcase;
